#include <iostream>
#include <wiringPi.h> // Include WiringPi library!

#include <vector>
#include <thread>
#include <mutex>
#include <atomic>
#include <bitset>
#include <cmath>
#include <iostream>
using namespace std;
// Pin number declarations. Use WiringPi chip pin numbers.
#define iA 8
#define iB 9
#define iC 7
#define iD 0

#define oA 15
#define oB 16
#define oC 1
#define oD 4

#define RW 2

#define SIGNAL_SLEEP 1
#define PRIO_DIFF 1
#define THREAD_SLEEP 1 //micros

struct Pin {
	int inn, out;
	Pin(int inn, int out) :inn(inn), out(out) {
		pinMode(inn, INPUT);
		pinMode(out, OUTPUT);
	}
	void signal() {
		digitalWrite(out, HIGH);
		digitalWrite(out, LOW);
	}
};

class Pins {
	std::vector<Pin> gpio_pins;
	Pin read_write;
	const int data_tab[7] = { 3,5,6,10,11,12,13 };
public:
	Pins() : read_write(RW, RW) {
		digitalWrite(RW, LOW);
		pullUpDnControl(RW, PUD_DOWN);

		gpio_pins.push_back(Pin(iA, oA));
		gpio_pins.push_back(Pin(iB, oB));
		gpio_pins.push_back(Pin(iC, oC));
		gpio_pins.push_back(Pin(iD, oD));
		for (auto& pin : gpio_pins) {
			pinMode(pin.inn, INPUT);
			pinMode(pin.out, OUTPUT);
			pullUpDnControl(pin.inn, PUD_DOWN);//PUD_OFF;PUD_DOWN;PUD_UP
			pullUpDnControl(pin.out, PUD_DOWN);
			digitalWrite(pin.inn, LOW);
			digitalWrite(pin.out, LOW);
		}

	}

	Pin operator[](int n) {

		return gpio_pins[n];
	}
	void writeData(uint8_t word) {
		auto t = std::bitset<7>(word).to_string();
		for (int i = 0; i < 7; i++) {
			digitalWrite(data_tab[i], t[i] == '1' ? HIGH : LOW);

		}
		read_write.signal();
	}
};

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


void bigWhile() {
	Pins system_pins;
	while (1) {
		for (int i = 0; i < 4; ++i) {
			if (digitalRead(system_pins[i].inn)) {
				system_pins[i].signal();
			}
		}
		std::this_thread::sleep_for(std::chrono::microseconds(THREAD_SLEEP));
	}
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void set_thread_priority(std::thread& trad, int prio) {
	sched_param sch;
	int policy = SCHED_RR;
	auto thread_ = trad.native_handle();
	pthread_getschedparam(thread_, &policy, &sch);
	sch.sched_priority = prio;
	pthread_setschedparam(thread_, policy, &sch);
}

void set_cpu(int cpu_core) {
	cpu_set_t mask;
	int status;

	CPU_ZERO(&mask);
	CPU_SET(cpu_core, &mask);
	status = sched_setaffinity(0, sizeof(mask), &mask);
}


static volatile bool running;
static volatile bool barrier;

void test_thread(Pin p) {


	while (barrier) {}
	static int thread_nr = 0;
	//set_cpu(thread_nr); //use this to put each thread on new core
	thread_nr = (thread_nr + 1) % 4;
	while (running) {
		std::this_thread::yield();
		if (digitalRead(p.inn))
		{
			p.signal();
		}
	}
}

void threaded() {
	Pins pinner;
	std::vector<std::thread> threads;
	for (int i = 0; i <4; ++i)
		threads.push_back(std::thread(test_thread, pinner[i]));
	int prio = 10;
	//for (auto& i : threads)
	//set_thread_priority(i, prio);
	barrier = false;
	char a;
	std::cin >> a; //press any key to exit
	running = false;
	for (int i = 0; i < 4; ++i)
		threads[i].join();
}
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void ansA() {
	digitalWrite(oA, HIGH);
	digitalWrite(oA, LOW);
}
void ansB() {
	digitalWrite(oB, HIGH);
	digitalWrite(oB, LOW);
}
void ansC() {
	digitalWrite(oC, HIGH);
	digitalWrite(oC, LOW);
}
void ansD() {
	digitalWrite(oD, HIGH);
	digitalWrite(oD, LOW);
}
void inter_driven() {
	Pins pinner;
	wiringPiISR(iA, INT_EDGE_RISING, &ansA);
	wiringPiISR(iB, INT_EDGE_RISING, &ansB);
	wiringPiISR(iC, INT_EDGE_RISING, &ansC);
	wiringPiISR(iD, INT_EDGE_RISING, &ansD);
	while (1) {
		std::this_thread::sleep_for(std::chrono::microseconds(1));
	}

}
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


std::mutex mut;

void tester(int define) {
	Pins pinner;
	//set_cpu(0);
	while (barrier) {};
	while (1) {
		mut.lock();
		pinner.writeData(define);
		mut.unlock();
		std::this_thread::yield();
	}
}
void noise() {
	set_cpu(0);
	int b = 0, c = 0;
	while (barrier) {};

	while (1) {
		//std::this_thread::yield();

		++b;
		if (b == 100) {
			cout << "100\n";
			b = 0;
		}

	}
}

void timeSection() {
	Pins p;

	while (1) {
		p.writeData(1);
		std::this_thread::yield();
	}

}

void timeCode() {
	std::vector<std::thread> threads;



	for (int i = 0; i < 4; i++) {
		threads.push_back(std::thread(tester, std::pow(2, i)));
	}
	barrier = false;
	for (int i = 0; i < 4; ++i)
		threads[i].join();

}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

int main()
{
	// Setup stuff:
	if (wiringPiSetup() < 0)
	{
		fprintf(stderr, "setup failed\n");
	}
	running = true;
	barrier = true;
	bigWhile();
	//threaded();
	//inter_driven();
	//timeCode();
	//timeSection();
	return 0;
}