import sys
INT=1
ACK=2
FD=[3,5,9,13,20,63] #seperation points in a line 
FL=16			#number of chars in a line
N_PINS=4
class dataLine():
	''' 
	Strict-like class for holding one line of test data from the system.
	'''
	def __init__(self,binary):
		
		self.b=binary
		
		self.b=self.b[2:]
		self.type=int(		self.b[FD[0]:FD[1]],2)
		self.inter_status=	self.b[FD[1]:FD[2]]
		self.ack_status=	self.b[FD[2]:FD[3]]
		self.inter_id=int(	self.b[FD[3]:FD[4]],2)
		self.timestamp=int(	self.b[FD[4]:FD[5]],2)
	def __str__(self):
		return str(self.type)+" "+str(self.inter_status)+" "+str(self.ack_status)+" "+str(self.timestamp)
		
class datafile():
	'''
	Class holding all the datalines of the test file. 
	It can calculate the response times of the logfile.
	'''
	def __init__(self,n):
		self.lines=[0]*n
		self.n=0
	def addLine(self,line):
		self.lines[self.n]=line
		self.n+=1
	def find_next_ack(self,j,inter):
		for i in range(j,self.n,1):
			if self.lines[i].ack_status[inter]=='1':
				return i
		return -1
	def response_times_intAck(self):
		res={0:[],1:[],2:[],3:[]}
		for l in range(N_PINS):
			prev_int=0
			for i in range(self.n):
				if 	(i==0 and self.lines[i].inter_status[l]=='1') 	or\
				 		(self.lines[i].inter_status[l]=='1' 	and\
				  		self.lines[i-1].inter_status[l]=='0'):
					pos=self.find_next_ack(i,l)
					if pos>=0:
						resTime=self.lines[pos].timestamp-self.lines[i].timestamp
						res[l].append(resTime)
		return res

class RTTresults():
	'''
	Main class for holding info about a test.
	Includes the entire test file and methods 
	for printing info and basic statistics. 
	'''
	def __init__(self,filename):
		self.filename=filename
		self.fil,self.n=self.__openFile(filename)
		self.responses=self.__parse()
	def __parse(self):
		df=datafile(self.n)
		for linje in self.fil:
			binary=bin(linje)
			if int(binary[2])>0:
				dl=dataLine(binary)
				df.addLine(dl)
		return df.response_times_intAck()
		
	def basicAnalyze(self):
		retDict={"Max":[],"Min":[],"Avg":[]}
		for key,val in self.responses.items():
			maxi=max(val) if len(val) else 0
			mini=min(val) if len(val) else 0
			avg=sum(val)/len(val) if len(val) else 0
			retDict["Max"].append(maxi)
			retDict["Min"].append(mini)
			retDict["Avg"].append(avg)
		return retDict
	def printInfo(self):
		print self.responses
		print self.basicAnalyze()	
	def __openFile(self,filename):
		file =open(filename,"r")

		text=file.read().strip()
		text=text.replace('\n','')
		text=text.replace('\r','')
		n_lines= len(text)/FL #FL=chars in one line
		lines=[0]*n_lines
		j=0
		for i in range(0,len(text),FL):
			lines[j]=int(text[i:i+FL],FL)
			j+=1
		return lines,n_lines

def main():
        
	if len(sys.argv)==2 and sys.argv[1][-3:]=="rts":
                        data=RTTresults(sys.argv[1])
                        data.printInfo()
        elif(len(sys.argv)==3 and sys.argv[1]=="folder"):
                import glob
                files= glob.glob(sys.argv[2]+"\*.rts")
                tot_data={"Max":[0]*N_PINS,"Min":[0]*N_PINS,"Avg":[0]*N_PINS}
                fileA=open(sys.argv[2]+"\A.txt","w")
                fileB=open(sys.argv[2]+"\B.txt","w")
                fileC=open(sys.argv[2]+"\C.txt","w")
                fileD=open(sys.argv[2]+"\D.txt","w")
                for fil in files:
                        raw=RTTresults(fil)
                        
                        for line in raw.responses[0]:
                                fileA.write(str(line*20*3)+'\n')
                        for line in raw.responses[1]:
                                fileB.write(str(line*20*3)+'\n')
                        for line in raw.responses[2]:
                                fileC.write(str(line*20*3)+'\n')
                        for line in raw.responses[3]:
                                fileD.write(str(line*20*3)+'\n')

                        data=raw.basicAnalyze()
                        for key in data:
                                for pin in range(N_PINS):
                                        
                                        tot_data[key][pin]+=data[key][pin]
                for key in tot_data:
                        for pin in range(N_PINS):
                                tot_data[key][pin]/=len(files)
                print tot_data
        
	else:
		print "Please input a filename"
if __name__=="__main__":
	main()
